@@ -561,6 +561,9 @@ gdk_directfb_window_new (GdkWindow              *parent,
       gdk_directfb_event_windows_add (window);
     }
 
+  if (attributes_mask & GDK_WA_TYPE_HINT)
+    gdk_window_set_type_hint (window, attributes->type_hint);
+
   return window;
 }
 
