@@ -85,14 +85,15 @@ typedef enum
  */
 typedef enum
 {
-  GDK_WA_TITLE	  = 1 << 1,
-  GDK_WA_X	  = 1 << 2,
-  GDK_WA_Y	  = 1 << 3,
-  GDK_WA_CURSOR	  = 1 << 4,
-  GDK_WA_COLORMAP = 1 << 5,
-  GDK_WA_VISUAL	  = 1 << 6,
-  GDK_WA_WMCLASS  = 1 << 7,
-  GDK_WA_NOREDIR  = 1 << 8
+  GDK_WA_TITLE	   = 1 << 1,
+  GDK_WA_X	   = 1 << 2,
+  GDK_WA_Y	   = 1 << 3,
+  GDK_WA_CURSOR	   = 1 << 4,
+  GDK_WA_COLORMAP  = 1 << 5,
+  GDK_WA_VISUAL	   = 1 << 6,
+  GDK_WA_WMCLASS   = 1 << 7,
+  GDK_WA_NOREDIR   = 1 << 8,
+  GDK_WA_TYPE_HINT = 1 << 9
 } GdkWindowAttributesType;
 
 /* Size restriction enumeration.
@@ -127,15 +128,20 @@ typedef enum
 {
   GDK_WINDOW_TYPE_HINT_NORMAL,
   GDK_WINDOW_TYPE_HINT_DIALOG,
-  GDK_WINDOW_TYPE_HINT_MENU,
+  GDK_WINDOW_TYPE_HINT_MENU,		/* Torn off menu */
   GDK_WINDOW_TYPE_HINT_TOOLBAR,
   GDK_WINDOW_TYPE_HINT_SPLASHSCREEN,
   GDK_WINDOW_TYPE_HINT_UTILITY,
   GDK_WINDOW_TYPE_HINT_DOCK,
-  GDK_WINDOW_TYPE_HINT_DESKTOP
+  GDK_WINDOW_TYPE_HINT_DESKTOP,
+  GDK_WINDOW_TYPE_HINT_DROPDOWN_MENU,	/* A drop down menu (from a menubar) */
+  GDK_WINDOW_TYPE_HINT_POPUP_MENU,	/* A popup menu (from right-click) */
+  GDK_WINDOW_TYPE_HINT_TOOLTIP,
+  GDK_WINDOW_TYPE_HINT_NOTIFICATION,
+  GDK_WINDOW_TYPE_HINT_COMBO,
+  GDK_WINDOW_TYPE_HINT_DND
 } GdkWindowTypeHint;
 
-
 /* The next two enumeration values current match the
  * Motif constants. If this is changed, the implementation
  * of gdk_window_set_decorations/gdk_window_set_functions
@@ -208,6 +214,7 @@ struct _GdkWindowAttr
   gchar *wmclass_name;
   gchar *wmclass_class;
   gboolean override_redirect;
+  GdkWindowTypeHint type_hint;
 };
 
 struct _GdkGeometry
@@ -287,6 +294,7 @@ struct _GdkWindowObject
 
   guint accept_focus : 1;
   guint focus_on_map : 1;
+  guint shaped : 1;
   
   GdkEventMask event_mask;
 };
@@ -395,6 +403,18 @@ void gdk_window_set_child_shapes (GdkWindow *window);
  */
 void gdk_window_merge_child_shapes (GdkWindow *window);
 
+void gdk_window_input_shape_combine_mask   (GdkWindow *window,
+					    GdkBitmap *mask,
+					    gint       x,
+					    gint       y);
+void gdk_window_input_shape_combine_region (GdkWindow *window,
+                                            GdkRegion *shape_region,
+                                            gint       offset_x,
+                                            gint       offset_y);
+void gdk_window_set_child_input_shapes     (GdkWindow *window);
+void gdk_window_merge_child_input_shapes   (GdkWindow *window);
+
+
 /*
  * Check if a window has been shown, and whether all its
  * parents up to a toplevel have been shown, respectively.
@@ -435,8 +455,10 @@ void	      gdk_window_set_hints	 (GdkWindow	  *window,
 					  gint		   max_height,
 					  gint		   flags);
 #endif
-void          gdk_window_set_type_hint    (GdkWindow       *window,
-					   GdkWindowTypeHint hint);
+void              gdk_window_set_type_hint (GdkWindow        *window,
+                                            GdkWindowTypeHint hint);
+GdkWindowTypeHint gdk_window_get_type_hint (GdkWindow        *window);
+
 void          gdk_window_set_modal_hint   (GdkWindow       *window,
                                            gboolean         modal);
 
