@@ -87,7 +87,7 @@ gtk_curve_get_type (void)
 
   if (!curve_type)
     {
-      static const GTypeInfo curve_info =
+      const GTypeInfo curve_info =
       {
 	sizeof (GtkCurveClass),
 	NULL,		/* base_init */
@@ -100,7 +100,7 @@ gtk_curve_get_type (void)
 	(GInstanceInitFunc) gtk_curve_init,
       };
 
-      curve_type = g_type_register_static (GTK_TYPE_DRAWING_AREA, "GtkCurve",
+      curve_type = g_type_register_static (GTK_TYPE_DRAWING_AREA, I_("GtkCurve"),
 					   &curve_info, 0);
     }
   return curve_type;
@@ -164,7 +164,7 @@ gtk_curve_class_init (GtkCurveClass *class)
 						       GTK_PARAM_READWRITE));
 
   curve_type_changed_signal =
-    g_signal_new ("curve_type_changed",
+    g_signal_new (I_("curve_type_changed"),
 		   G_OBJECT_CLASS_TYPE (gobject_class),
 		   G_SIGNAL_RUN_FIRST,
 		   G_STRUCT_OFFSET (GtkCurveClass, curve_type_changed),
@@ -427,7 +427,6 @@ gtk_curve_graph_events (GtkWidget *widget, GdkEvent *event, GtkCurve *c)
 {
   GdkCursorType new_type = c->cursor_type;
   gint i, src, dst, leftbound, rightbound;
-  GdkEventButton *bevent;
   GdkEventMotion *mevent;
   GtkWidget *w;
   gint tx, ty;
@@ -481,7 +480,6 @@ gtk_curve_graph_events (GtkWidget *widget, GdkEvent *event, GtkCurve *c)
     case GDK_BUTTON_PRESS:
       gtk_grab_add (widget);
 
-      bevent = (GdkEventButton *) event;
       new_type = GDK_TCROSS;
 
       switch (c->curve_type)
@@ -715,7 +713,7 @@ gtk_curve_set_curve_type (GtkCurve *c, GtkCurveType new_type)
 	  gtk_curve_interpolate (c, width, height);
 	}
       g_signal_emit (c, curve_type_changed_signal, 0);
-      g_object_notify (G_OBJECT (c), "curve_type");
+      g_object_notify (G_OBJECT (c), "curve-type");
       gtk_curve_draw (c, width, height);
     }
 }
@@ -789,7 +787,7 @@ gtk_curve_reset (GtkCurve *c)
   if (old_type != GTK_CURVE_TYPE_SPLINE)
     {
        g_signal_emit (c, curve_type_changed_signal, 0);
-       g_object_notify (G_OBJECT (c), "curve_type");
+       g_object_notify (G_OBJECT (c), "curve-type");
     }
 }
 
@@ -836,19 +834,19 @@ gtk_curve_set_range (GtkCurve *curve,
   g_object_freeze_notify (G_OBJECT (curve));
   if (curve->min_x != min_x) {
      curve->min_x = min_x;
-     g_object_notify (G_OBJECT (curve), "min_x");
+     g_object_notify (G_OBJECT (curve), "min-x");
   }
   if (curve->max_x != max_x) {
      curve->max_x = max_x;
-     g_object_notify (G_OBJECT (curve), "max_x");
+     g_object_notify (G_OBJECT (curve), "max-x");
   }
   if (curve->min_y != min_y) {
      curve->min_y = min_y;
-     g_object_notify (G_OBJECT (curve), "min_y");
+     g_object_notify (G_OBJECT (curve), "min-y");
   }
   if (curve->max_y != max_y) {
      curve->max_y = max_y;
-     g_object_notify (G_OBJECT (curve), "max_y");
+     g_object_notify (G_OBJECT (curve), "max-y");
   }
   g_object_thaw_notify (G_OBJECT (curve));
 
@@ -894,7 +892,7 @@ gtk_curve_set_vector (GtkCurve *c, int veclen, gfloat vector[])
   if (old_type != GTK_CURVE_TYPE_FREE)
     {
        g_signal_emit (c, curve_type_changed_signal, 0);
-       g_object_notify (G_OBJECT (c), "curve_type");
+       g_object_notify (G_OBJECT (c), "curve-type");
     }
 
   gtk_curve_draw (c, c->num_points, height);
