@@ -54,7 +54,7 @@ static void     gtk_radio_button_get_property   (GObject             *object,
 						 GValue              *value,
 						 GParamSpec          *pspec);
 
-G_DEFINE_TYPE (GtkRadioButton, gtk_radio_button, GTK_TYPE_CHECK_BUTTON);
+G_DEFINE_TYPE (GtkRadioButton, gtk_radio_button, GTK_TYPE_CHECK_BUTTON)
 
 static guint group_changed_signal = 0;
 
@@ -449,21 +449,23 @@ gtk_radio_button_focus (GtkWidget         *widget,
       GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
       GtkWidget *new_focus = NULL;
 
-      focus_list = g_slist_copy (radio_button->group);
-      
       switch (direction)
 	{
-	case GTK_DIR_TAB_FORWARD:
-	case GTK_DIR_TAB_BACKWARD:
-	  return FALSE;
 	case GTK_DIR_LEFT:
 	case GTK_DIR_RIGHT:
+	  focus_list = g_slist_copy (radio_button->group);
 	  focus_list = g_slist_sort_with_data (focus_list, left_right_compare, toplevel);
 	  break;
 	case GTK_DIR_UP:
 	case GTK_DIR_DOWN:
+	  focus_list = g_slist_copy (radio_button->group);
 	  focus_list = g_slist_sort_with_data (focus_list, up_down_compare, toplevel);
 	  break;
+	case GTK_DIR_TAB_FORWARD:
+	case GTK_DIR_TAB_BACKWARD:
+          /* fall through */
+	default:
+	  return FALSE;
 	}
 
       if (direction == GTK_DIR_LEFT || direction == GTK_DIR_UP)
