@@ -494,22 +494,24 @@ gtk_tree_path_new_from_indices (gint first_index,
 gchar *
 gtk_tree_path_to_string (GtkTreePath *path)
 {
-  gchar *retval, *ptr;
-  gint i;
+  gchar *retval, *ptr, *end;
+  gint i, n;
 
   g_return_val_if_fail (path != NULL, NULL);
 
   if (path->depth == 0)
     return NULL;
 
-  ptr = retval = g_new0 (gchar, path->depth*8);
-  g_sprintf (retval, "%d", path->indices[0]);
-  while (*ptr != '\000')
+  n = path->depth * 12;
+  ptr = retval = g_new0 (gchar, n);
+  end = ptr + n;
+  g_snprintf (retval, end - ptr, "%d", path->indices[0]);
+  while (*ptr != '\000') 
     ptr++;
 
   for (i = 1; i < path->depth; i++)
     {
-      g_sprintf (ptr, ":%d", path->indices[i]);
+      g_snprintf (ptr, end - ptr, ":%d", path->indices[i]);
       while (*ptr != '\000')
 	ptr++;
     }
@@ -773,7 +775,8 @@ gtk_tree_path_is_descendant (GtkTreePath *path,
  * gtk_tree_path_next:
  * @path: A #GtkTreePath.
  *
- * Moves the @path to point to the next node at the current depth.
+ * Moves the @path to point to the next potential node at the current depth.
+ * You must use gtk_tree_model_get_iter() to discover whether a corresponding row actually exists.
  **/
 void
 gtk_tree_path_next (GtkTreePath *path)
